# Copyright (c) 2012 Cloudera, Inc. All rights reserved.

import os
import shutil
import simplejson

from cmf.monitor.json_metrics_utils import ParseError
import cmf.monitor.flume.json_parser
from StringIO import StringIO
from cmf.monitor import MonitorDaemon, schema, constants
from cmf.monitor.flume import FlumeAgentMonitor
from cmf.monitor.tests import make_config_file

from nose.tools import assert_true, assert_equals, assert_false

class MockedFlumeAgentMonitor(FlumeAgentMonitor):
  def __init__(self, daemon, registry, string):
    FlumeAgentMonitor.__init__(self, daemon, registry)
    self.set_string(string)
    self.set_throw_error(None)

  def set_string(self, string):
    self._string = string

  def set_throw_error(self, error):
    self._error = error

  def urlopen(self, url):
    if self._error:
      raise self._error
    return StringIO(self._string)

def test_flume_agent_lifecycle():
  daemon = MonitorDaemon("host", "host", agent=None)
  reg = daemon.get_registry_for_testing()
  monitor = MockedFlumeAgentMonitor(daemon, reg, \
      cmf.monitor.flume.json_parser.SAMPLE)
  monitor.refresh_pid(os.getpid(), True)

  properties = """[agent]
role_name = %s
service_name = my_flume1
service_version = 4
host = localhost
log_dir = /path/to/logs
collect_interval = 1.0
"""

  # Make two different configs
  conf_dir = make_config_file(properties % ("flume-agent-test",))
  conf_dir2 = make_config_file(properties % ("flume-agent",))

  try:
    assert_equals(0, len(reg.get_registry()))

    # Start it
    monitor.refresh(conf_dir)
    assert_equals(monitor, reg.get_registry().pop())
    monitor.collect()
    role_update = monitor.get_role_update().to_avro_dict()
    assert_equals('flume-agent-test', role_update['rolename'])

    # Refresh with a different config
    monitor.refresh(conf_dir2)
    assert_equals(monitor, reg.get_registry().pop())
    assert_equals(1, len(reg.get_registry()))

    monitor.collect()

    # Test metrics and role update
    role_update = monitor.get_role_update().to_avro_dict()

    assert_equals('flume-agent', role_update['rolename'])
    assert_equals('my_flume1', role_update['service_name'])
    metric_dict = { }
    for metric_obj in role_update['metrics']:
      metric_dict[metric_obj['id']] = metric_obj['value']
    assert_true(metric_dict[schema.TOTAL_CPU_USER] >= 0.0)
    assert_true(metric_dict[schema.TOTAL_CPU_SYSTEM] >= 0.0)

    dir_metrics = role_update['dirUpdates']
    assert_true(len(dir_metrics) == 1, "We configured 1 dirs: %s" % dir_metrics)
    assert_true(os.path.ismount(dir_metrics[0]['mountpoint']))
    assert_equals('/path/to/logs', dir_metrics[0]['path'])

    # Test when the process is not running
    monitor.refresh_pid(None)
    monitor.collect()

    role_update = monitor.get_role_update().to_avro_dict()

    assert_equals('flume-agent', role_update['rolename'])
    assert_equals('my_flume1', role_update['service_name'])
    metric_dict = { }
    for metric_obj in role_update['metrics']:
      metric_dict[metric_obj['id']] = metric_obj['value']

    dir_metrics = role_update['dirUpdates']
    assert_true(len(dir_metrics) == 1, "We configured 1 dirs: %s" % dir_metrics)
    assert_true(os.path.ismount(dir_metrics[0]['mountpoint']))
    assert_equals('/path/to/logs', dir_metrics[0]['path'])

    # Test with a collection error
    monitor.refresh_pid(os.getpid(), True)
    monitor.set_throw_error(IOError("BOOM!"))
    monitor.collect()
    monitor.clear_last_reported_timestamp()
    role_update = monitor.get_role_update().to_avro_dict()
    metric_dict = { }
    for metric_obj in role_update['metrics']:
      metric_dict[metric_obj['id']] = metric_obj['value']
    assert_true(metric_dict[schema.TOTAL_CPU_USER] >= 0.0)
    assert_equals(constants.WEB_METRICS_COLLECTION_COMMUNICATION_FAILURE,
                  metric_dict[schema.WEB_METRICS_COLLECTION_STATUS])
    assert_true(schema.WEB_METRICS_COLLECTION_DURATION in metric_dict)

    # Test with some invalid JSON
    monitor.set_throw_error(None)
    monitor.set_string("THIS IS BOGUS JSON!")
    monitor.collect()
    monitor.clear_last_reported_timestamp()
    role_update = monitor.get_role_update().to_avro_dict()
    metric_dict = { }
    for metric_obj in role_update['metrics']:
      metric_dict[metric_obj['id']] = metric_obj['value']
    assert_true(metric_dict[schema.TOTAL_CPU_USER] >= 0.0)
    assert_equals(constants.WEB_METRICS_COLLECTION_PARSE_FAILURE,
                  metric_dict[schema.WEB_METRICS_COLLECTION_STATUS])
    assert_true(schema.WEB_METRICS_COLLECTION_DURATION in metric_dict)

    # Stop it
    monitor.stop()
    assert_equals(0, len(reg.get_registry()))
  finally:
    shutil.rmtree(conf_dir)
    shutil.rmtree(conf_dir2)

def test_metrics_parsing():
  def expect_parse_error(expected_msg, config):
    msg = None
    try:
      cmf.monitor.flume.json_parser.JsonParser( \
          simplejson.load(StringIO(config)))
    except ParseError, e:
      msg = str(e)
    assert_equals(expected_msg, msg)

  jp = cmf.monitor.flume.json_parser.JsonParser( \
      simplejson.load(StringIO(cmf.monitor.flume.json_parser.SAMPLE)))
  assert_equals(cmf.monitor.flume.json_parser.SAMPLE_CORRECT_UPDATES, \
      jp.updates)
  # No components; not an error.
  expect_parse_error(None, "{}")
  expect_parse_error("Missing type for component 'foo'", r"""{
    "foo": {}
  }""")
  expect_parse_error("Expected 'source.' prefix in component name 'foo'", r"""{
    "foo": {"Type":"SOURCE"}
  }""")
  expect_parse_error("Expected 'source.' prefix in component name 'SINK.foo'", r"""{
    "SINK.foo": {"Type":"SOURCE"}
  }""")
  expect_parse_error("No component name after type prefix 'SOURCE.'", r"""{
    "SOURCE.": {"Type":"SOURCE"}
  }""")
  # Unknown component type; not an error (for forwards compatibility).
  expect_parse_error(None, r"""{
    "BAR.foo": {"Type":"BAR"}
  }""")
  # Unknown metric (SomeFutureMetric); not an error (for forwards compatibility).
  expect_parse_error(None, r"""{
    "CHANNEL.channel1":{
       "EventPutSuccessCount":"10000",
       "Type":"CHANNEL",
       "StopTime":"0",
       "EventPutAttemptCount":"10011",
       "ChannelSize":"10000",
       "StartTime":"1345147577389",
       "EventTakeSuccessCount":"0",
       "ChannelCapacity":"10000",
       "SomeFutureMetric":"1234",
       "EventTakeAttemptCount":"0"
    }
  }""")
  expect_parse_error("Flume metric 'EventTakeSuccessCount' not found for component 'channel1'", r"""{
    "CHANNEL.channel1":{
       "EventPutSuccessCount":"10000",
       "Type":"CHANNEL",
       "StopTime":"0",
       "EventPutAttemptCount":"10011",
       "ChannelSize":"10000",
       "StartTime":"1345147577389",
       "ChannelCapacity":"10000",
       "EventTakeAttemptCount":"0"
    }
  }""")
