# Copyright (c) 2011 Cloudera, Inc. All rights reserved.

import os
import shutil
try:
  from cStringIO import StringIO
except:
  from StringIO import StringIO

from cmf.monitor import MonitorDaemon, schema, constants
from cmf.monitor.stoppable_thread import StoppableThread
from cmf.monitor.tasktracker import TaskTrackerMonitor, json_parser
from cmf.monitor.tasktracker.tasktracker import MockTaskTracker
from cmf.monitor.tasktracker.tasktracker import UNKNOWN_ENTITY
from cmf.monitor.tests import make_config_file
from nose.tools import assert_true, assert_equals

class MockTaskTrackerMonitor(TaskTrackerMonitor):
  def __init__(self, daemon, registry, attempt_monitor, string):
    TaskTrackerMonitor.__init__(self, daemon, registry, attempt_monitor)
    self.set_string(string)
    self.set_throw_error(None)

  def set_string(self, string):
    self._string = string

  def set_throw_error(self, error):
    self._error = error

  def urlopen(self, url):
    if self._error:
      raise self._error
    return StringIO(self._string)

def do_nothing():
  pass

# CM 3.7 config file format.
def _make_cm37_format_config_file(port):
  properties = """[tasktracker]
role_name = mapreduce-TT-1
service_name = my_mapreduce
host = localhost
collect_interval = 1.0
tasktracker_instrumentation_url = http://127.0.0.1:%s
task_tracker_http_address = 127.0.0.1
task_tracker_http_port = 50035
task_tracker_mapred_local_dir = /path/to/dir,/another/data/dir
"""
  return  make_config_file(properties % (port,))

# CM 4.1 config file: we added the log directory.
def _make_cm41_format_config_file(port):
  properties = """[tasktracker]
role_name = mapreduce-TT-1
service_name = my_mapreduce
host = localhost
log_dir = /path/to/logs
collect_interval = 1.0
tasktracker_instrumentation_url = http://127.0.0.1:%s
task_tracker_http_address = 127.0.0.1
task_tracker_http_port = 50035
task_tracker_mapred_local_dir = /path/to/dir,/another/data/dir
"""
  return  make_config_file(properties % (port,))

def test_tasktracker_lifecycle():
  do_tasktracker_lifecycle(_make_cm37_format_config_file, 2)
  do_tasktracker_lifecycle(_make_cm41_format_config_file, 3)

def do_tasktracker_lifecycle(make_config_file_function,
                             expected_directory_metrics):
  daemon = MonitorDaemon("host", "host", agent=None)
  reg = daemon.get_registry_for_testing()
  do_nothing_attempt_monitor = \
    StoppableThread("DoNothingAttemptMonitor", 5.0, do_nothing)
  monitor = MockTaskTrackerMonitor(daemon, reg, do_nothing_attempt_monitor,
                                   json_parser.SAMPLE)
  monitor.refresh_pid(os.getpid(), True)

  # Make two different configs
  conf_dir = make_config_file_function(1234)
  conf_dir2 = make_config_file_function(5678)

  try:
    assert_equals(0, len(reg.get_registry()))

    # Start it
    monitor.refresh(conf_dir)
    assert_equals(monitor, reg.get_registry().pop())
    monitor.collect()
    monitor.collect_attempt_metrics(MockTaskTracker('mock_job1', 0, 16, 1, 300, False))
    assert_true('1234' in monitor.instrumentation_url)

    # Refresh with a different config
    monitor.refresh(conf_dir2)
    assert_equals(monitor, reg.get_registry().pop())
    assert_equals(1, len(reg.get_registry()))

    monitor.collect()
    assert_true('5678' in monitor.instrumentation_url)

    # Test metrics and role update
    monitor.clear_last_reported_timestamp()
    role_update = monitor.get_role_update().to_avro_dict()
    assert_equals('mapreduce-TT-1', role_update['rolename'])
    assert_equals('my_mapreduce', role_update['service_name'])
    metric_dict = { }
    for metric_obj in role_update['metrics']:
      metric_dict[metric_obj['id']] = metric_obj['value']
    assert_equals(1, metric_dict[schema.JVM_GC_COUNT])
    assert_true(metric_dict[schema.TOTAL_CPU_USER] >= 0.0)
    assert_true(metric_dict[schema.TOTAL_CPU_SYSTEM] >= 0.0)
    assert_equals(1, metric_dict[schema.JVM_BLOCKED_THREADS])
    assert_equals(2, metric_dict[schema.JVM_NEW_THREADS])
    assert_equals(3, metric_dict[schema.JVM_RUNNABLE_THREADS])
    assert_equals(4, metric_dict[schema.JVM_TERMINATED_THREADS])
    assert_equals(5, metric_dict[schema.JVM_TIMED_WAITING_THREADS])
    assert_equals(6, metric_dict[schema.JVM_WAITING_THREADS])
    assert_equals(21, metric_dict[schema.JVM_TOTAL_THREADS])
    assert_equals(constants.WEB_METRICS_COLLECTION_GOOD,
                  metric_dict[schema.WEB_METRICS_COLLECTION_STATUS])
    assert_true(schema.WEB_METRICS_COLLECTION_DURATION in metric_dict)

    dir_metrics = role_update['dirUpdates']
    assert_true(len(dir_metrics) == expected_directory_metrics,
                "We configured %d dirs: %s" % (expected_directory_metrics, dir_metrics))
    assert_true(os.path.ismount(dir_metrics[0]['mountpoint']))
    assert_true(dir_metrics[0]['path'] is not None)

    attempt_updates = monitor.get_attempt_updates()
    assert_true(len(attempt_updates) != 0)
    assert_true('metrics' in attempt_updates[0].keys())
    assert_true(len(attempt_updates[0]['metrics']) != 0)
    assert_true('id' in attempt_updates[0]['metrics'][0].keys())
    assert_true(attempt_updates[0]['metrics'][0]['id'] is not None)
    assert_true('value' in attempt_updates[0]['metrics'][0].keys())

    # Test that we do not generate any updates with defaults
    monitor.collect_attempt_metrics(MockTaskTracker('mock_job2', 0, 16, 1, 300, True))
    attempt_updates = monitor.get_attempt_updates()
    for attempt_update in attempt_updates:
      assert_true(attempt_update['context']['job_id'] != UNKNOWN_ENTITY)

    # Test when the process is not running
    monitor.refresh_pid(None)
    monitor.collect()
    monitor.collect_attempt_metrics(MockTaskTracker('mock_job1', 0, 16, 1, 300, False))

    monitor.clear_last_reported_timestamp()
    role_update = monitor.get_role_update().to_avro_dict()

    assert_equals('mapreduce-TT-1', role_update['rolename'])
    assert_equals('my_mapreduce', role_update['service_name'])
    metric_dict = { }
    for metric_obj in role_update['metrics']:
      metric_dict[metric_obj['id']] = metric_obj['value']

    dir_metrics = role_update['dirUpdates']
    assert_true(len(dir_metrics) == expected_directory_metrics,
                "We configured %d dirs: %s" % (expected_directory_metrics, dir_metrics))
    assert_true(os.path.ismount(dir_metrics[0]['mountpoint']))
    assert_true(dir_metrics[0]['path'] is not None)

    assert_equals(0, len(monitor.get_attempt_updates()))

    # Test with a collection error
    monitor.refresh_pid(os.getpid(), True)
    monitor.set_throw_error(IOError("BOOM!"))
    monitor.collect()
    monitor.clear_last_reported_timestamp()
    role_update = monitor.get_role_update().to_avro_dict()
    metric_dict = { }
    for metric_obj in role_update['metrics']:
      metric_dict[metric_obj['id']] = metric_obj['value']
    assert_true(metric_dict[schema.TOTAL_CPU_USER] >= 0.0)
    assert_equals(constants.WEB_METRICS_COLLECTION_COMMUNICATION_FAILURE,
                  metric_dict[schema.WEB_METRICS_COLLECTION_STATUS])
    assert_true(schema.WEB_METRICS_COLLECTION_DURATION in metric_dict)

    # Test with some invalid JSON
    monitor.set_throw_error(None)
    monitor.set_string("THIS IS BOGUS JSON!")
    monitor.collect()
    monitor.clear_last_reported_timestamp()
    role_update = monitor.get_role_update().to_avro_dict()
    metric_dict = { }
    for metric_obj in role_update['metrics']:
      metric_dict[metric_obj['id']] = metric_obj['value']
    assert_true(metric_dict[schema.TOTAL_CPU_USER] >= 0.0)
    assert_equals(constants.WEB_METRICS_COLLECTION_PARSE_FAILURE,
                  metric_dict[schema.WEB_METRICS_COLLECTION_STATUS])
    assert_true(schema.WEB_METRICS_COLLECTION_DURATION in metric_dict)

    # Stop it
    monitor.stop()
    assert_equals(0, len(reg.get_registry()))
  finally:
    shutil.rmtree(conf_dir)
    shutil.rmtree(conf_dir2)
